/* filo.c - xmain */

#include <conf.h>
#include <kernel.h>
#include <tty.h>
#include <sleep.h>

#define EOF	-2

int	phil();
int	snum();
int	clkmgr();
int	disk();

/*----------------------------------------------------------------------------
 * xmain - user process started up at initialization
 *----------------------------------------------------------------------------
 */

/* this program creates 4 windows:
 *	1 - a real-time clock in the upper right hand corner
 *	2 - The dining philosophers in the top left window
 *	3 - The sleeping digits in the top right window
 *	4 - Messages in the bottom window
 */

xmain()
{
	resume(create(phil,INITSTK,INITPRIO,"thinkers",0));
	resume(create(snum,INITSTK,INITPRIO,"digits",0));
	resume(create(clkmgr,INITSTK,INITPRIO,"clock",0));
	resume(create(disk,INITSTK,INITPRIO,"demoprint",0));
}

/* DINING PHILOSOPHERS */

#define fork1  fork[i]
#define fork2  fork[(i+1) % 5]

int	wp;			/* philosophers window			*/
int	room;
int	fork[5];
int	prsem;			/* printing mutual exclusion semaph.	*/

phil()
{
	int philosopher();
	int i;
	char *pn="phil ";

    wp = open(getdev("tty"),"#0,1:30,15","wht/blu"); /* yellow on blue */
	room = screate(4);
	prsem = screate(1);
	for (i=0; i<5; i++)
		fork[i]=screate(1);
	for (i=0; i<5; i++) {
		*(pn+4)='0'+i;
		resume(create(philosopher,INITSTK,INITPRIO,pn,1,i));
	}
}

philosopher(i)
int i;
{
	while(1) {
		hungry(i);
		wait(room);
		wait(fork1);
		wait(fork2);
		eat(i);
		signal(fork1);
		signal(fork2);
		signal(room);
		think(i);
	}
}

hungry(i)
int i;
{
	wait(prsem);
	fprintf(wp,"\nPhilosopher %d is hungry.",i);
	signal(prsem);
}

eat(i)
int i;
{
	int time=9;	

	wait(prsem);
	fprintf(wp,"\nPhilosopher %d is eating.",i);
	signal(prsem);
	sleep(time);
}

think(i)
int i;
{
	int time=6;	

	wait(prsem);
	fprintf(wp,"\nPhilosopher %d is thinking.",i);
	signal(prsem);
	sleep(time);
}

/* SLEEPING DIGITS */

int	wn;

snum()
{
	int	w;
	int i;
	char *dn = "digit ";
	int digit();

	wn = open(getdev("tty"),"#32,1:79,15","grn"); /* green on black */
	resume(create(digit,INITSTK,INITPRIO,"digit0",2,1,'.'));
	for (i = 1 ; i < 6 ; i++) {
		*(dn + 5) = '0' + i;
	      	resume(create(digit, INITSTK, INITPRIO, dn, 2,i*10,'0'+i));
     	}
}


digit(i,c)
int i;
char c;
{
	while (1) {
        	sleept(i);
	        putc(wn, c);
	}
}

#define	SECPERHR	3600
#define	SECPERMIN	60
#define TP1080SEC	19663
#define	TPCLKT		1080

/* REAL-TIME CLOCK DISPLAY */
clkmgr()
{
	int	ps;
	int	hours, mins, secs, h, w;
	long	t;
	int	sys_pcxget();

	w = open(getdev("tty"),"70,0:79,0",NULL);
	while (1) {
		disable(ps);
		t = tod;
		restore(ps);
		t = (t/TP1080SEC) * TPCLKT + ((t % TP1080SEC) * TPCLKT) /
		     TP1080SEC;
		hours = (t / SECPERHR);
		mins  = (t %= SECPERHR)/SECPERMIN;
		secs  = (t %= SECPERMIN);
		fprintf(w,"\n%02d:%02d:%02d",hours,mins,secs);
		sleep(1);
	}
}

char	buf[512];

disk()
{
	int	w;		/* display window			*/
	int	fd;		/* file descriptor			*/
	int	ch;
	char	*fgets();

	if ( (fd=open(getdev("ds0"),"demo","ro")) == SYSERR )
		return;
    w = open(getdev("tty"),"#0,17:79,23","red");    /* red on black */
	for ( ; ; ) {
		while ( fgets(buf,sizeof(buf),fd) != NULL ) {
			buf[strlen(buf)-1] = '\0';
			if ( strcmp(buf,"%%") == 0 ) {
				sleep(15);
				fprintf(w,"\n\n\n\n\n");
			} else
				fprintf(w,"\n%s",buf);
		}
		seek(fd,0L);
	}
}

char *fgets(s, n, stream)
char 	*s;
int	n;
int	stream;
{
	int	i;
	register int	ch;
	register char	*cp;

	cp = s;
	--n;
	for ( i=0; i<n; i++ ) {
		if ( (ch=getc(stream)) == EOF || ch == SYSERR ) {
			*cp = '\0';
			return( (i==0) ? NULL : s );
		}
		*cp++ = ch;
		if ( ch == '\n' ) {
			*cp = '\0';
			return(s);
		}
	}
	*cp = '\0';
	return(s);
}
