/**************************************************************
*   bestand:   DEMO.C
*   taal:      Turbo C 2.0 / PC-Xinu
*   functie:   PC-Xinu demonstratie programma
*   auteur:    L.H. Meuldijk
***************************************************************/
/* INCLUDES */

#include <alias.h>
#include <time.h>
/**************************************************************/
#define    LF      10        /* ascii-waarde linefeed */
#define    BUFLEN  256       /* comm.buffer lengte    */
#define    SPD     10        /* boodschap suspend     */
#define    REJ     13        /* bdschp 'verworpen'    */
/**************************************************************/

int buffer[BUFLEN];

int shlpid,
    prodpid,
    conspid;

SEMAPHORE  produced,
           consumed;
time_t t;

int pids(int pid)
{   if (pid == shlpid)  return OK;
    if (pid == prodpid) return OK;
    if (pid == conspid) return OK;
    return SYSERR;
}

int        flush(int),
           which(int);
void       shlerror(int, int),
           toonhelp(int);
PROCESS    xmain(void),
           xclock(void),
           shell(void),
           producer(void),
           consumer(void);

PROCESS xmain()
{
    srand((unsigned) time(&t));
    consumed=screate(BUFLEN);
    produced=screate(0);
    resume(shlpid = create(shell,    STDSTK, 10, "shell", 0));
    resume(prodpid =create(producer, STDSTK, 10, "prod",  0));
    resume(conspid =create(consumer, STDSTK, 10, "cons",  0));
}

PROCESS producer()
{   int i = 0, prodwin, product, r;
    prodwin = open(CONSOLE,"#40,0:79,10","red");
    for (;;) {
      if (recvclr() != OK) {
	    fprintf(prodwin,"%d SUSPENDED\n",prodpid);
	    suspend(prodpid);
      }
      wait(consumed);
      product = rand();
      buffer[i%BUFLEN] = product;
      signal(produced);
      r = rand() % 4;
      fprintf(prodwin, " P(%d) produced %d  %5d  %d\n", prodpid,i,product,r);
      sleept(r);
      i++;
    }
}

PROCESS consumer()
{   int i = 0, product, conswin, r;
    conswin = open(CONSOLE,"#40,11:79,21","grn");
    for (;;) {
      if (recvclr() != OK) {
	    fprintf(conswin,"%d SUSPENDED\n",conspid);
	    suspend(conspid);
      }
      wait(produced);
      product = buffer[i%BUFLEN];
      signal(consumed);
      r = rand() % 4;
      fprintf(conswin, " P(%d) consumed %d  %5d  %d\n", conspid,i,product,r);
      sleept(r);
      i++;
    }
}

PROCESS shell()
{   char ch;
    int pid, ret, shlwin;
    shlwin = open(CONSOLE,"#0,0:39,24","wht/blu");
    control(shlwin,SETCUR);
    fprintf(shlwin,"PC-Xinu Demonstratie-Programma\n");
    fprintf(shlwin,"tik h als u hulp wilt\n");
    while (1) {
        switch (recvclr()){
            case REJ: shlerror(shlwin, 3); break;
            default:  ;
        }
	ch = getc(shlwin);
        
	 switch (ch) {
	    case 's':
		if ((pid = which(shlwin)) == SYSERR) {
		    shlerror(shlwin, 2); break;
		}
		if (pids(pid)==SYSERR || sendf(pid, SPD)==SYSERR)
		    send(shlpid, REJ);
		break;
	    case 'r':
		if ((pid = which(shlwin)) == SYSERR) {
		    shlerror(shlwin, 2); break;
		}
		if (resume(pid) == SYSERR)
		    sendf(shlpid, REJ);
		else{
		    fprintf(shlwin,"Proces %2d herstart \n",pid);
		}
		break;
	    case 'h':
		toonhelp(shlwin); break;
	    case 'x':
		fprintf(shlwin,"\nTERUG NAAR DOS\n");
		sleep(1);
		kill(prodpid); kill(conspid);
		close(CONSOLE); xdone();
	    default:
		if ((int)ch != LF) {
		    flush(shlwin);
		    shlerror(shlwin, 1);
		}
	}
    }
}

int which(int win)
{   char ch[3];
    int i, ret;
    ret = OK;
    for (i=0;i<2;i++)
    if ((ch[i] = getc(win)) < '0' || (ch[i] > '9')) ret = SYSERR;
    ch[2] = 0x00;
    return ((ret==SYSERR) ? SYSERR : atoi(ch));
}

void shlerror(int win, int i)
{   switch (i) {
	case 1: fprintf(win, "GEEN COMMANDO\n");
		break;
	case 2: fprintf(win, "ONJUISTE OPERAND\n");
		break;
	case 3: fprintf(win, "COMMANDO GENEGEERD\n");
		break;
     }
}

int flush(int win)
{   char ch;
    int  i = 0;
    do {
        ch = getc(win);
        i++;
    } while ((int) ch != LF);
    return (i == 1 ? OK : SYSERR);
}

void toonhelp(int win)
{   fprintf(win,"\nBeschikbare commando's:\n");
    fprintf(win,"-------------------\n");
    fprintf(win,"sID = schortop proces met pid ID\n");
    fprintf(win,"rID = herstart proces met pid ID\n");
    fprintf(win,"h   = toon hulpinfo\n");
    fprintf(win,"x   = exit PC-XINU\n");
}
